@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusT.c
  *
- * Copyright 2009  David A. Bagley, bagleyd@tux.org
+ * Copyright 2009 - 2010  David A. Bagley, bagleyd@tux.org
  *
  * All rights reserved.
  *
@@ -35,7 +35,7 @@ static void simpleParser(AbacusWidget w, char * buffer,
 	int numberCount = 0, decimalCount = 0;
 	Boolean digit = False, decimal = False, negate = False;
 	int decimalPosition = w->abacus.decimalPosition;
-	
+
 	if (w->abacus.decks[BOTTOM].piece != 0)
 		decimalPosition--;
 	if (w->abacus.decimalPosition >= w->abacus.shiftPercent &&
@@ -67,19 +67,19 @@ static void simpleParser(AbacusWidget w, char * buffer,
 		} else if (buffer[i] == '.' && !decimal) {
 			decimal = True;
 			if (numberCount == 0)
-				(void) sprintf(aString, "%s%c", aString, '.');
+				(void) strcat(aString, ".");
 			else
-				(void) sprintf(bString, "%s%c", bString, '.');
+				(void) strcat(bString, ".");
 		} else if (IS_DIGIT(buffer[i])) {
 			if (!decimal || decimalCount < decimalPosition) {
 				if (numberCount == 0) {
 					if (!digit && negate)
-						(void) sprintf(aString, "%s-", aString);
-					(void) sprintf(aString, "%s%c", aString, buffer[i]);
+						(void) strcat(aString, "-");
+					aString = strAddChar(aString, buffer[i]);
 				} else {
 					if (!digit && negate)
-						(void) sprintf(bString, "%s-", bString);
-					(void) sprintf(bString, "%s%c", bString, buffer[i]);
+						(void) strcat(bString, "-");
+					bString = strAddChar(bString, buffer[i]);
 				}
 				negate = False;
 			}
@@ -381,11 +381,11 @@ static void decimalSafe(char * aString, char * bString)
 
 	aDecimal = getDecimalStringPosition(aString);
 	if (aDecimal == (int) strlen(aString)) {
-		(void) sprintf(aString, "%s%c", aString, '.');
+		(void) strcat(aString, ".");
 	}
 	bDecimal = getDecimalStringPosition(bString);
 	if (bDecimal == (int) strlen(bString)) {
-		(void) sprintf(bString, "%s%c", bString, '.');
+		(void) strcat(bString, ".");
 	}
 }
 
@@ -569,7 +569,7 @@ static void setStringBuffer(AbacusWidget w, char * string, int aux,
 		wap = (AbacusWidget *) w->abacus.leftAux;
 	else if (aux == 2 && w->abacus.rightAux != NULL)
 		wap = (AbacusWidget *) w->abacus.rightAux;
-	wa = *wap;	
+	wa = *wap;
 	topFactor = wa->abacus.decks[TOP].factor;
 	bottomNumber = wa->abacus.decks[BOTTOM].number;
 	if (bottomNumber <= wa->abacus.base / 2)
@@ -597,10 +597,10 @@ static void addSafe(char * aString, char * bString)
 
 	if (aCount > bCount) {
 		for (i = bCount; i < aCount; i++)
-			(void) sprintf(bString, "%s%c", bString, '0');
+			(void) strcat(bString, "0");
 	} else {
 		for (i = aCount; i < bCount; i++)
-			(void) sprintf(aString, "%s%c", aString, '0');
+			(void) strcat(aString, "0");
 	}
 	if (aDecimal > bDecimal) {
 		for (i = bDecimal; i < aDecimal; i++) {
@@ -631,7 +631,7 @@ static int nextRail(AbacusWidget w, int step, int aux, Boolean rightToLeft,
 		wap = (AbacusWidget *) w->abacus.leftAux;
 	else if (aux == 2 && w->abacus.rightAux != NULL)
 		wap = (AbacusWidget *) w->abacus.rightAux;
-	wa = *wap;	
+	wa = *wap;
 	shiftPercent = wa->abacus.shiftPercent;
 	piece = checkPiece(wa);
 	piecePercent = checkPiecePercent(wa);
@@ -711,7 +711,7 @@ static int nextCharPosition(AbacusWidget w, int step, int carryStep, int aux,
 		wap = (AbacusWidget *) w->abacus.leftAux;
 	else if (aux == 2 && w->abacus.rightAux != NULL)
 		wap = (AbacusWidget *) w->abacus.rightAux;
-	wa = *wap;	
+	wa = *wap;
 	piece = checkPiece(wa);
 	piecePercent = checkPiecePercent(wa);
 	subdeck = checkSubdeck(wa, 3);
@@ -953,7 +953,7 @@ static double rootApprox(double guess, double power, int n)
 		(void) printf("%d: %g %g %g\n", i, xn, xi, diff);
 #endif
 		if (diff < 0.0)
-			diff = -diff;  
+			diff = -diff;
 		if (diff < EPSILON)
 			break;
 		xi = xn;
@@ -980,7 +980,6 @@ static void testRoot(AbacusWidget w, double guess, int position, int root)
 #ifdef DEBUG
 	(void) printf("incr root = %g\n", c);
 #endif
-	
 }
 
 /* Figure out next intGroup value */
@@ -1200,45 +1199,63 @@ static void testCbrt(AbacusWidget w, int position)
 }
 #endif
 
+static void
+highlightRail(AbacusWidget w, int aux, int rail, Boolean highlight)
+{
+	AbacusWidget *wap = &w, wa;
+
+	if (aux == 1) {
+		if (w->abacus.leftAux == NULL)
+			return;
+		wap = (AbacusWidget *) w->abacus.leftAux;
+	} else if (aux == 2) {
+		if (w->abacus.rightAux == NULL)
+			return;
+		wap = (AbacusWidget *) w->abacus.rightAux;
+	}
+	wa = *wap;
+	drawBeadRail(wa, rail + wa->abacus.decimalPosition, highlight);
+}
+
 /* Tell about what is going to happen */
 static Boolean pendingUpdate(AbacusWidget w, char * buffer,
-	int line, int position, int bottomNumber)
+	int line, int aux, int position, int bottomNumber)
 {
 	Boolean done;
 
 	(void) sprintf(buffer, "For rail %d", position);
 	if (w->abacus.lower == 0 && w->abacus.upper == 0 &&
 			w->abacus.carry[w->abacus.state] == 0) {
-		(void) sprintf(buffer, "%s, do nothing", buffer);
+		(void) strcat(buffer, " do nothing");
 		w->abacus.step++; /* or else two do nothings */
 		done = True;
 	} else {
+		highlightRail(w, aux, position, True);
 		if (w->abacus.lower != 0) {
-			(void) sprintf(buffer, "%s, %s %d",
-				buffer,
-				((w->abacus.lower < 0) ? "take off" : "put on"),
-				ABS(w->abacus.lower));
-			if (bottomNumber <= DEFAULT_BASE / 2)
-				(void) sprintf(buffer, "%s %s lower deck",
-					buffer,
-					((w->abacus.lower < 0) ? "from" : "to"));
+			(void) strcat(buffer, ((w->abacus.lower < 0) ?
+				", take off " : ", put on "));
+			strAddInt(buffer, ABS(w->abacus.lower));
+			if (bottomNumber <= DEFAULT_BASE / 2) {
+				(void) strcat(buffer, ((w->abacus.lower < 0) ?
+					" from lower deck" : " to lower deck"));
+			}
 		}
 		if (w->abacus.upper != 0) {
-			  (void) sprintf(buffer, "%s, %s %d %s upper deck",
-			  	buffer,
-			  	((w->abacus.upper < 0) ? "take off" : "put on"),
-			  	ABS(w->abacus.upper),
-			  	((w->abacus.upper < 0) ? "from" : "to"));
-			}
+			(void) strcat(buffer, ((w->abacus.upper < 0) ?
+				", take off " : ", put on "));
+			strAddInt(buffer, ABS(w->abacus.upper));
+			(void) strcat(buffer, ((w->abacus.upper < 0) ?
+				" from upper deck" : " to upper deck"));
+		}
 		if (w->abacus.carry[w->abacus.state] != 0) {
-			(void) sprintf(buffer, "%s, %s %d (on next move)",
-				buffer,
-				((w->abacus.carry[w->abacus.state] > 0) ? "carry" : "borrow"),
-				ABS(w->abacus.carry[w->abacus.state]));
+			(void) strcat(buffer, ((w->abacus.state > 0) ?
+				", carry " : ", borrow "));
+			strAddInt(buffer, ABS(w->abacus.carry[w->abacus.state]));
+			(void) strcat(buffer, " (on next move)");
 		}
 		done = False;
 	}
-	(void) sprintf(buffer, "%s.", buffer);
+	(void) strcat(buffer, ".");
 	drawLineText(w, buffer, line);
 	return done;
 }
@@ -1325,7 +1342,7 @@ static Boolean nextPositionProduct(AbacusWidget w,
 	if (w->abacus.carryStep == 0) {
 		/* 2 digits * 2 step display */
 		if (((n % 2 == 0) && rightToLeft) ||
-		    ((n % 2 == 1) && !rightToLeft)) {
+				((n % 2 == 1) && !rightToLeft)) {
 			*bValue = (*aDigit * *bDigit) % w->abacus.base;
 		} else {
 			*bValue = (*aDigit * *bDigit) / w->abacus.base;
@@ -1374,7 +1391,7 @@ static Boolean nextPositionDivision(AbacusWidget w,
 
 		if (w->abacus.rightAux != NULL)
 			wap = (AbacusWidget *) w->abacus.rightAux;
-		wa = *wap;	
+		wa = *wap;
 		if (w->abacus.reg > 1)
 			(void) strcpy(w->abacus.cString, w->abacus.rString);
 		w->abacus.regCount++;
@@ -1436,7 +1453,7 @@ static Boolean nextPositionDivision(AbacusWidget w,
 	if (w->abacus.carryStep == 0) {
 		/* 2 digits * 2 step display */
 		if (((n % 2 == 0) && rightToLeft) ||
-		    ((n % 2 == 1) && !rightToLeft)) {
+				((n % 2 == 1) && !rightToLeft)) {
 			*bValue = (w->abacus.qDigit * *bDigit) % w->abacus.base;
 		} else {
 			*bValue = (w->abacus.qDigit * *bDigit) / w->abacus.base;
@@ -1471,7 +1488,7 @@ static Boolean nextPositionRoot(AbacusWidget w, int root,
 
 		if (w->abacus.leftAux != NULL)
 			wap = (AbacusWidget *) w->abacus.leftAux;
-		wa = *wap;	
+		wa = *wap;
 		if (w->abacus.reg > 1)
 			(void) strcpy(w->abacus.cString, w->abacus.rString);
 		w->abacus.regCount++;
@@ -1558,13 +1575,13 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			free(w->abacus.sString);
 		if ((w->abacus.aString = (char *) malloc(sizeof (char) *
 				w->abacus.numDigits + 2)) &&
-		   (w->abacus.bString = (char *) malloc(sizeof (char) *
+			(w->abacus.bString = (char *) malloc(sizeof (char) *
 				w->abacus.numDigits + 2)) &&
-		   (w->abacus.cString = (char *) malloc(sizeof (char) *
+			(w->abacus.cString = (char *) malloc(sizeof (char) *
 				w->abacus.numDigits + 2)) &&
-		   (w->abacus.rString = (char *) malloc(sizeof (char) *
+			(w->abacus.rString = (char *) malloc(sizeof (char) *
 				w->abacus.numDigits + 2)) &&
-		   (w->abacus.sString = (char *) malloc(sizeof (char) *
+			(w->abacus.sString = (char *) malloc(sizeof (char) *
 				w->abacus.numDigits + 2)))
 #endif
 			w->abacus.step++;
@@ -1649,7 +1666,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
 			if (!w->abacus.lee) {
-				(void) sprintf(buffer1, "%s%s", buffer1,
+				(void) strcat(buffer1,
 					" ... works best with lee option");
 			}
 			drawLineText(w, buffer1, 0);
@@ -1688,7 +1705,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
 			if (!w->abacus.lee) {
-				(void) sprintf(buffer1, "%s%s", buffer1,
+				(void) strcat(buffer1,
 					" ... works best with lee option");
 			}
 			drawLineText(w, buffer1, 0);
@@ -1715,7 +1732,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				((w->abacus.op == 'v') ? "Square" : "Cube"),
 				w->abacus.aString);
 			if (!w->abacus.lee) {
-				(void) sprintf(buffer1, "%s%s", buffer1,
+				(void) strcat(buffer1,
 					" ... works best with lee option");
 			}
 			drawLineText(w, buffer1, 0);
@@ -1755,21 +1772,22 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
 			if (w->abacus.carryStep == 0) {
-				char buf[120]; 
+				char buf[120];
 
 				convertFromInteger(buf, base, aDigit);
-				(void) sprintf(buffer1,
-					"%s ... %s", buffer1, buf);
+				(void) strcat(buffer1, " ... ");
+				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, bDigit);
-				(void) sprintf(buffer1,
-					"%s %c %s", buffer1, w->abacus.op, buf);
+				(void) strcat(buffer1, " ");
+				(void) strAddChar(buffer1, w->abacus.op);
+				(void) strcat(buffer1, " ");
+				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, aDigit + bDigit);
-				(void) sprintf(buffer1,
-					"%s = %s", buffer1, buf);
+				(void) strcat(buffer1, " = ");
+				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, bValue);
 			} else {
-				(void) sprintf(buffer1, "%s ... carrying %d",
-					buffer1, 1);
+				(void) strcat(buffer1, " ... carrying 1");
 			}
 			drawLineText(w, buffer1, 0);
 		} else if (w->abacus.op == '*') {
@@ -1779,23 +1797,23 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				w->abacus.aString, w->abacus.op,
 				w->abacus.bString);
 			if (w->abacus.carryStep == 0) {
-				char buf[120]; 
+				char buf[120];
 
 				convertFromInteger(buf, base, aDigit);
-				(void) sprintf(buffer1,
-					"%s ... %s", buffer1, buf);
+				(void) strcat(buffer1, " ... ");
+				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, bDigit);
-				(void) sprintf(buffer1,
-					"%s * %s", buffer1, buf);
+				(void) strcat(buffer1, " * ");
+				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, aDigit * bDigit);
-				(void) sprintf(buffer1,
-					"%s = %s", buffer1, buf);
+				(void) strcat(buffer1, " = ");
+				(void) strcat(buffer1, buf);
 				convertFromInteger(buf, base, bValue);
-				(void) sprintf(buffer1,
-					"%s, adding %s digit", buffer1, buf);
+				(void) strcat(buffer1, ", adding ");
+				(void) strcat(buffer1, buf);
+				(void) strcat(buffer1, " digit");
 			} else {
-				(void) sprintf(buffer1, "%s ... carrying %d",
-					buffer1, 1);
+				(void) strcat(buffer1, " ... carrying 1");
 			}
 			drawLineText(w, buffer1, 0);
 		} else if (w->abacus.op == '/') {
@@ -1809,53 +1827,54 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				char buf[120];
 
 				convertFromInteger(buf, base, w->abacus.qDigit);
-				(void) sprintf(buffer1,
-					"%s ... %s", buffer1, buf);
+				(void) strcat(buffer1, " ... ");
+				(void) strcat(buffer1, buf);
 				convertFromDouble(buf, base, w->abacus.divisor);
-				(void) sprintf(buffer1,
-					"%s * %s", buffer1, buf);
+				(void) strcat(buffer1, " * ");
+				(void) strcat(buffer1, buf);
 				convertFromDouble(buf, base,
 					w->abacus.divisor * w->abacus.qDigit);
-				(void) sprintf(buffer1,
-					"%s = %s", buffer1, buf);
+				(void) strcat(buffer1, " = ");
+				(void) strcat(buffer1, buf);
 				if (w->abacus.reg >= 0) {
 					AbacusWidget *wap = &w, wa;
 					int bn;
 
 					if (w->abacus.rightAux != NULL)
 						wap = (AbacusWidget *) w->abacus.rightAux;
-					wa = *wap;	
+					wa = *wap;
 					bn = wa->abacus.decks[BOTTOM].number;
 					convertFromInteger(buf, base,
 						w->abacus.qDigit);
-					(void) sprintf(buffer1,
-						"%s, register %s on second auxiliary, rail %d",
-						buffer1, buf, w->abacus.qPosition);
+					(void) strcat(buffer1, ", register ");
+					(void) strcat(buffer1, buf);
+					(void) strcat(buffer1, " on second auxiliary, rail ");
+					(void) strAddInt(buffer1, w->abacus.qPosition);
 					drawLineText(w, buffer1, 0);
 					w->abacus.reg++;
 					(void) pendingUpdate(w, buffer2, 1,
-						w->abacus.qPosition, bn);
+						2, w->abacus.qPosition, bn);
 					return;
 				}
 				if (w->abacus.divisor != bDigit) {
 					convertFromInteger(buf, base, w->abacus.qDigit);
-					(void) sprintf(buffer1,
-						"%s ... %s", buffer1, buf);
+					(void) strcat(buffer1, " ... ");
+					(void) strcat(buffer1, buf);
 					convertFromInteger(buf, base, bDigit);
-					(void) sprintf(buffer1,
-						"%s * %s", buffer1, buf);
+					(void) strcat(buffer1, " * ");
+					(void) strcat(buffer1, buf);
 					convertFromInteger(buf, base,
 						w->abacus.qDigit * bDigit);
-					(void) sprintf(buffer1,
-						"%s = %s", buffer1, buf);
+					(void) strcat(buffer1, " = ");
+					(void) strcat(buffer1, buf);
 				}
 				convertFromInteger(buf, base, bValue);
-				(void) sprintf(buffer1,
-					"%s, subtracting %s digit", buffer1, buf);
+				(void) strcat(buffer1, ", subtracting ");
+				(void) strcat(buffer1, buf);
+				(void) strcat(buffer1, " digit");
 				drawLineText(w, buffer1, 0);
 			} else {
-				(void) sprintf(buffer1, "%s ... borrowing %d",
-					buffer1, 1);
+				(void) strcat(buffer1, " ... borrowing 1");
 			}
 			drawLineText(w, buffer1, 0);
 		} else if (w->abacus.op == 'v' || w->abacus.op == 'u') {
@@ -1865,14 +1884,16 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			(void) sprintf(buffer1, "Taking %s Root of %s",
 				(w->abacus.op == 'v') ? "Square" : "Cube", w->abacus.cString);
 			if (w->abacus.carryStep == 0) {
-				(void) sprintf(buffer1, "%s ... %d",
-					buffer1, w->abacus.qDigit);
-				if (w->abacus.op == 'u')
-					(void) sprintf(buffer1, "%s * %d",
-						buffer1, w->abacus.qDigit);
-				(void) sprintf(buffer1, "%s * %d = %d",
-					buffer1, w->abacus.qDigit,
-					(w->abacus.qDigit * w->abacus.qDigit *
+				(void) strcat(buffer1, " ... ");
+				(void) strAddInt(buffer1, w->abacus.qDigit);
+				if (w->abacus.op == 'u') {
+					(void) strcat(buffer1, " * ");
+					(void) strAddInt(buffer1, w->abacus.qDigit);
+				}
+				(void) strcat(buffer1, " * ");
+				(void) strAddInt(buffer1, w->abacus.qDigit);
+				(void) strcat(buffer1, " = ");
+				(void) strAddInt(buffer1, (w->abacus.qDigit * w->abacus.qDigit *
 					((w->abacus.op == 'v') ? 1 : w->abacus.qDigit)));
 				if (w->abacus.reg >= 0) {
 					AbacusWidget *wap = &w, wa;
@@ -1880,27 +1901,28 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 
 					if (w->abacus.leftAux != NULL)
 						wap = (AbacusWidget *) w->abacus.leftAux;
-					wa = *wap;	
+					wa = *wap;
 					bn = wa->abacus.decks[BOTTOM].number;
-					(void) sprintf(buffer1,
-						"%s, register %d on first auxiliary, rail %d",
-						buffer1, w->abacus.qDigit, w->abacus.qPosition);
+					(void) strcat(buffer1, " , register  ");
+					(void) strAddInt(buffer1, w->abacus.qDigit);
+					(void) strcat(buffer1, " on first auxiliary, rail ");
+					(void) strAddInt(buffer1, w->abacus.qPosition);
 					drawLineText(w, buffer1, 0);
 					w->abacus.reg++;
 					(void) pendingUpdate(w, buffer2, 1,
-						w->abacus.qPosition, bn);
+						1, w->abacus.qPosition, bn);
 					return;
 				}
-				(void) sprintf(buffer1, "%s ... subtracting %d digit",
-					buffer1, bValue);
+				(void) strcat(buffer1, " ... subtracting ");
+				(void) strAddInt(buffer1, bValue);
+				(void) strcat(buffer1, " digit");
 				drawLineText(w, buffer1, 0);
 			} else {
-				(void) sprintf(buffer1, "%s ... borrowing %d",
-					buffer1, 1);
+				(void) strcat(buffer1, " ... borrowing 1");
 			}
 			drawLineText(w, buffer1, 0);
 		}
-		if (!pendingUpdate(w, buffer2, 1, rPosition, bottomNumber))
+		if (!pendingUpdate(w, buffer2, 1, 0, rPosition, bottomNumber))
 			done = False;
 		if (w->abacus.carry[w->abacus.state] == 0 &&
 				w->abacus.carryStep == 0 && done) {
@@ -1951,6 +1973,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 					if (w->abacus.upper != 0)
 						setAbacusMove(w, ACTION_MOVE, 2, 1, w->abacus.qPosition,
 							w->abacus.upper);
+					highlightRail(w, 2, w->abacus.qPosition, False);
 				}
 				setStringBuffer(w, w->abacus.sString, 2,
 					w->abacus.qPosition,
@@ -1980,6 +2003,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 					if (w->abacus.upper != 0)
 						setAbacusMove(w, ACTION_MOVE, 1, 1, w->abacus.qPosition,
 							w->abacus.upper);
+					highlightRail(w, 1, w->abacus.qPosition, False);
 				}
 				setStringBuffer(w, w->abacus.sString, 1,
 					w->abacus.qPosition,
@@ -2004,6 +2028,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 		if (w->abacus.upper != 0)
 			setAbacusMove(w, ACTION_MOVE, 0, 1, rPosition,
 				w->abacus.upper);
+		highlightRail(w, 0, rPosition, False);
 #if 0
 		if (w->abacus.carry[w->abacus.state] != 0) {
 			setAbacusMove(w, ACTION_MOVE, 0, 0, rPosition + 1,
@@ -2041,7 +2066,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 					w->abacus.rString);
 			}
 			drawLineText(w, buffer3, 2);
-	 		if ((done && w->abacus.rightToLeftAdd) ||
+			if ((done && w->abacus.rightToLeftAdd) ||
 					!w->abacus.rightToLeftAdd) {
 				if (w->abacus.carry[w->abacus.state] != 0) {
 					if (w->abacus.carryStep == 0) {
